function [x] = GaussianElimination_Pivoting(A,b)

n = length(b);

for k=1:n-1
    % Find largest pivot in column (in or beneath row k)
    [pivotVal, pivotRow] = max(abs(A(k:n,k)));
    pivotRow = pivotRow+k-1;
    
    % Interchange rows of matrix and RHS vector if pivot is beneath row k
    if (pivotRow ~= k)
        A([k pivotRow], :) = A([pivotRow k], :);
        b([k pivotRow]) = b([pivotRow k]);
    end
    
    % Apply transformation to remaining submatrix and RHS vector
    % (same as in GaussianElimination)
    for i=k+1:n
        m = A(i,k)/A(k,k); % multipliyer for current row i
        for j=k:n % loop to update row i
            A(i,j) = A(i,j) - m*A(k,j);
        end
        b(i) = b(i) - m*b(k); % update RHS
    end   
end

% A is now upper triangular.  Use backsubstitution to solve the transformed problem.
x = BackSubstitution(A,b);

