function [x,u] = convDiffReact(f,a,b,N,ua,ub,alpha,beta,gamma)
% Solve the convection-diffusion-reaction equation with
% function f, interval [a,b] discretized with N points

% discretize the interval, h is the point spacing
x = linspace(a,b,N)';
h = (b-a)/(N-1);
    
% setup matrix for u'' (will have N-2 unknowns)
e = ones(N-2,1);
A = spdiags([(alpha/(h^2) - beta/(2*h))*e, ...
    (-2*alpha/(h^2) + gamma)*e,...
    (alpha/(h^2)+ beta/(2*h))*e], -1:1, N-2, N-2);

% setup rhs vector
b = feval(f,x(2:end-1));
b(1) = b(1)-alpha*ua/(h^2) + beta*ua/(2*h);
b(end)=b(end)-alpha*ub/(h^2) - beta*ub/(2*h);
    
% solve the linear system
utemp = A \ b;
    
% put solution together to include boundary points, and so it
% correponds to x's at same index
u = [ua;utemp;ub];
end
