function y = forwardEuler(func,t,y1)
% Solve the ODE y'=f(t,y) with initial condition y(t1)=y1 and return
% the function values as a vector.
% func is the function handle for f(t,y) and t is a vector
% of the times: [t1,t2,...,tn].

% N is the total number of points, initialize y to be the same
% size as t:
N = length(t);
y = zeros(N,1);

% Set initial condition:
y(1)=y1;

% use forward Euler to find y(i+1) using y(i)
for i=1:N-1
    y(i+1) = y(i) + ( t(i+1) - t(i) ) * func(t(i),y(i));
end

end
